@@ -21,6 +21,9 @@
  */
 
 #include "config.h"
+
+#include <string.h>
+
 #include <glib/gi18n-lib.h>
 
 #include <libsoup/soup.h>
@@ -467,6 +470,7 @@ static gchar *
 normalize_uri (const gchar *address, gchar **server)
 {
   SoupURI *uri;
+  const gchar *path;
   gchar *scheme;
   gchar *ret;
   gchar *uri_string;
@@ -509,20 +513,33 @@ normalize_uri (const gchar *address, gchar **server)
   else if (g_strcmp0 (scheme, "davs") == 0)
     soup_uri_set_scheme (uri, SOUP_URI_SCHEME_HTTPS);
 
+  path = soup_uri_get_path (uri);
+  if (!g_str_has_suffix (path, "/"))
+    {
+      gchar *new_path;
+
+      new_path = g_strconcat (path, "/", NULL);
+      soup_uri_set_path (uri, new_path);
+      path = soup_uri_get_path (uri);
+      g_free (new_path);
+    }
+
   if (server != NULL)
     {
-      const gchar *path;
       gchar *port_string;
+      gchar *pretty_path;
       guint port;
 
-      path = soup_uri_get_path (uri);
-      if (g_strcmp0 (path, "/") == 0)
-        path = "";
-
       port = soup_uri_get_port (uri);
       port_string = g_strdup_printf (":%u", port);
-      *server = g_strconcat (soup_uri_get_host (uri), (port == std_port) ? "" : port_string, path, NULL);
+
+      pretty_path = g_strdup (path);
+      pretty_path[strlen(pretty_path) - 1] = '\0';
+
+      *server = g_strconcat (soup_uri_get_host (uri), (port == std_port) ? "" : port_string, pretty_path, NULL);
+
       g_free (port_string);
+      g_free (pretty_path);
     }
 
   ret = soup_uri_to_string (uri, FALSE);
@@ -758,18 +775,21 @@ add_account (GoaProvider    *provider,
   username = gtk_entry_get_text (GTK_ENTRY (data.username));
   password = gtk_entry_get_text (GTK_ENTRY (data.password));
 
+  uri = normalize_uri (uri_text, &server);
+  presentation_identity = g_strconcat (username, "@", server, NULL);
+
   /* See if there's already an account of this type with the
    * given identity
    */
   provider_type = goa_provider_get_provider_type (provider);
   if (!goa_utils_check_duplicate (client,
                                   username,
+                                  presentation_identity,
                                   provider_type,
                                   (GoaPeekInterfaceFunc) goa_object_peek_password_based,
                                   &data.error))
     goto out;
 
-  uri = normalize_uri (uri_text, &server);
   uri_webdav = g_strconcat (uri, WEBDAV_ENDPOINT, NULL);
   g_cancellable_reset (data.cancellable);
   goa_http_client_check (http_client,
@@ -822,6 +842,7 @@ add_account (GoaProvider    *provider,
       gtk_widget_set_no_show_all (data.cluebar, FALSE);
       gtk_widget_show_all (data.cluebar);
 
+      g_clear_pointer (&presentation_identity, g_free);
       g_clear_pointer (&server, g_free);
       g_clear_pointer (&uri, g_free);
       goto http_again;
@@ -844,7 +865,6 @@ add_account (GoaProvider    *provider,
    * can create a proxy for the new object) so run the mainloop while
    * waiting for this to complete
    */
-  presentation_identity = g_strconcat (username, "@", server, NULL);
   goa_manager_call_add_account (goa_client_get_manager (client),
                                 goa_provider_get_provider_type (provider),
                                 username,
